/****************************************************************************
 *
 * Project: Generic test routines.
 *
 * Copyright by Olimex Ltd. All rights reserved.
 *
 * File: extcon-conf.c
 * Description: External connector test configuration for STM32-LCD.
 * Developer: Dimitar Dimitrov ( dinuxbg,gmail.com )
 *
 * Last change: $Date$
 * Revision: $Revision: 29 $
 * Id: $Id$
 *
 ****************************************************************************/

#include <stdint.h>
#include <stdio.h>
#include <assert.h>

#include "extcon.h"
#include "utils.h"

#include "stm32f10x_lib.h"
#include "platform_config.h"


#define _C_PORTA		'A'
#define _C_PORTB		'B'
#define _C_PORTC		'C'
#define _C_PORTD		'D'
#define _C_PORTE		'E'
#define _C_PORTF		'F'
#define _C_PORTG		'G'

#define PIN(CON_PIN, PRT, PIN, LQFP_PIN)	\
		{ \
			.con_pin = CON_PIN, \
			.port_pin = ((_C_PORT ## PRT)<<8) | (PIN), \
			.chip_pin = LQFP_PIN \
		}

static void print_chip_pin(char *buf, const struct extcon_pin *pin)
{
	sprintf(buf, "%d", pin->chip_pin);
}


static void print_port_pin(char *buf, const struct extcon_pin *pin)
{
	sprintf(buf, "P%c%d", pin->port_pin>>8, pin->port_pin & 0xff);
}

static GPIO_TypeDef *get_port(const struct extcon_pin *pin)
{
	GPIO_TypeDef *port_base;

	switch(pin->port_pin >> 8) {
		case 'A':
			port_base = GPIOA;
			break;
		case 'B':
			port_base = GPIOB;
			break;
		case 'C':
			port_base = GPIOC;
			break;
		case 'D':
			port_base = GPIOD;
			break;
		case 'E':
			port_base = GPIOE;
			break;
		case 'F':
			port_base = GPIOF;
			break;
		case 'G':
			port_base = GPIOG;
			break;
		default:
			for(;;);
	}
	return port_base;
}


static int stm32_gpio_read(const struct extcon_pin *pin)
{
	uint32_t sr;
	uint32_t mask;
	GPIO_TypeDef *port_base = get_port(pin);
       
	mask = (1 << (pin->port_pin & 0xff) );
	sr = port_base->IDR;
	return sr & mask;
}


static void stm32_gpio_set_inp(const struct extcon_pin *pin)
{
	GPIO_TypeDef *port_base = get_port(pin);
	const uint32_t pin_mask = 1 << (pin->port_pin & 0xff);
	GPIO_InitTypeDef GPIO_InitStructure;

	GPIO_InitStructure.GPIO_Pin = pin_mask;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
	GPIO_Init(port_base, &GPIO_InitStructure);
}


static void stm32_gpio_set_outp(const struct extcon_pin *pin, int val)
{
	GPIO_TypeDef *port_base = get_port(pin);
	const uint32_t pin_mask = 1 << (pin->port_pin & 0xff);
	GPIO_InitTypeDef GPIO_InitStructure;

	GPIO_InitStructure.GPIO_Pin = pin_mask;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_Init(port_base, &GPIO_InitStructure);

	if(val) {
		GPIO_SetBits(port_base, pin_mask);
	} else {
		GPIO_ResetBits(port_base, pin_mask);
	}
}


static const struct extcon_pin uext1con_main_outp = PIN(9, A,5, 41);

static const struct extcon_pin uext1con_pins[] = {
	/* CONN_PIN   PORT_PIN  CHIP_PIN */
	PIN( 3,		A,9, 	101),
	PIN( 4,		A,10, 	102),
	PIN( 5,		B,6, 	136),
	PIN( 6,		B,7, 	137),
	PIN( 7,		A,6, 	42),
	PIN( 8,		A,7, 	43),
	PIN( 10,	A,4, 	40)
};


static const struct extcon_pin uext40con_main_outp = PIN(40, C,5, 45);

static const struct extcon_pin uext40con_pins_1[] = {
	/* CONN_PIN   PORT_PIN  CHIP_PIN */
	PIN( 3,		A,9, 	101),
	PIN( 4,		A,10, 	102),
	PIN( 5,		B,6, 	136),
	PIN( 6,		B,7, 	137),
	PIN( 7,		A,6, 	42),
	PIN( 8,		A,7, 	43),
	PIN( 9,		A,5, 	41),
	PIN( 15,	F,15, 	55),
	PIN( 16,	F,14, 	54),
	PIN( 17,	F,13, 	53),
	PIN( 18,	F,12, 	50),
	PIN( 19,	F,11, 	49),
	PIN( 20,	F,10, 	22),
	PIN( 21,	F,9, 	21),
	PIN( 22,	F,8, 	20),
	PIN( 23,	F,7, 	19),
	PIN( 24,	F,6, 	18),
	PIN( 25,	F,5, 	15),
	PIN( 26,	F,4, 	14),
	PIN( 27,	F,3, 	13),
	PIN( 28,	F,2, 	12),
	PIN( 29,	F,1, 	11),
	PIN( 30,	F,0, 	10),
	PIN( 34,	B,0, 	46),
	PIN( 36,	B,1, 	47),
	PIN( 37,	A,4, 	40),
	PIN( 38,	C,4, 	44)
};


static const struct extcon_pin uext40con_pins_2[] = {
	/* CONN_PIN   PORT_PIN  CHIP_PIN */
	PIN( 3,		A,9, 	101),
	PIN( 4,		A,10, 	102),
	PIN( 5,		B,6, 	136),
	PIN( 6,		B,7, 	137),
	PIN( 7,		A,6, 	42),
	PIN( 8,		A,7, 	43),
	PIN( 9,		A,5, 	41),
	PIN( 10,	A,4, 	40),
	PIN( 15,	F,15, 	55),
	PIN( 16,	F,14, 	54),
	PIN( 17,	F,13, 	53),
	PIN( 18,	F,12, 	50),
	PIN( 19,	F,11, 	49),
	PIN( 20,	F,10, 	22),
	PIN( 21,	F,9, 	21),
	PIN( 22,	F,8, 	20),
	PIN( 23,	F,7, 	19),
	PIN( 24,	F,6, 	18),
	PIN( 25,	F,5, 	15),
	PIN( 26,	F,4, 	14),
	PIN( 27,	F,3, 	13),
	PIN( 28,	F,2, 	12),
	PIN( 29,	F,1, 	11),
	PIN( 30,	F,0, 	10),
	PIN( 34,	B,0, 	46),
	PIN( 36,	B,1, 	47),
	PIN( 38,	C,4, 	44)
};



static const struct extcon_pin uext2con_main_outp = PIN(9, B,13, 74);

static const struct extcon_pin uext2con_pins[] = {
	/* CONN_PIN   PORT_PIN  CHIP_PIN */
	PIN( 3,		A,2, 	36),
	PIN( 4,		A,3, 	37),
	PIN( 5,		B,8, 	139),
	PIN( 6,		B,9, 	140),
	PIN( 7,		B,14, 	75),
	PIN( 8,		B,15, 	76),
	PIN( 10,	B,12, 	73)
};

static const struct extcon_pin extcon_main_outp = PIN(10, B,5, 135);

static const struct extcon_pin extcon_pins[] = {
	/* CONN_PIN   PORT_PIN  CHIP_PIN */
	PIN( 3,		E,0, 	141),
	PIN( 4,		E,1, 	142),
	PIN( 5,		E,5, 	4),
	PIN( 6,		E,6, 	5),
	PIN( 7,		C,6, 	96),
	PIN( 8,		C,7, 	97),
	PIN( 9,		C,13, 	7),
	PIN( 15,	G,15, 	132),
	PIN( 16,	G,14, 	129),
	PIN( 17,	G,13, 	128),
	PIN( 18,	G,12, 	127),
	PIN( 19,	G,11, 	126),
	PIN( 20,	G,10, 	125),
	PIN( 21,	G,9, 	124),
	PIN( 22,	G,8, 	93),
	PIN( 23,	G,7, 	92),
	PIN( 24,	G,6, 	91),
	PIN( 25,	G,5, 	90),
	PIN( 26,	G,4, 	89),
	PIN( 27,	G,3, 	88),
	PIN( 28,	G,2, 	87),
	PIN( 29,	G,1, 	57),
	PIN( 30,	G,0, 	56),
	PIN( 34,	D,6, 	122),
	PIN( 35,	D,12, 	81),
	PIN( 36,	D,11, 	80),
	PIN( 37,	B,2, 	48),
	PIN( 39,	A,1, 	35),
	PIN( 40,	A,8, 	100)
};



static const struct extcon_jack extcons_phase1[] = {
	{
		.name = "UEXT1",
		.main_outp_pin = &uext1con_main_outp,
		.pins = uext1con_pins,
		.num_pins = ARRAY_NUMELEM(uext1con_pins)
	},
	{
		.name = "UEXT2",
		.main_outp_pin = &uext2con_main_outp,
		.pins = uext2con_pins,
		.num_pins = ARRAY_NUMELEM(uext2con_pins)
	},
	{
		.name = "EXT",
		.main_outp_pin = &extcon_main_outp,
		.pins = extcon_pins,
		.num_pins = ARRAY_NUMELEM(extcon_pins)
	}
};


static const struct extcon_jack extcons_phase2[] = {
	{
		.name = "UEXT40 testA",
		.main_outp_pin = &uext40con_main_outp,
		.pins = uext40con_pins_1,
		.num_pins = ARRAY_NUMELEM(uext40con_pins_1)
	},
	{
		.name = "UEXT40 testB",
		.main_outp_pin = &uext40con_main_outp,
		.pins = uext40con_pins_2,
		.num_pins = ARRAY_NUMELEM(uext40con_pins_2)
	}
};


/** Provide a sufficient time for pin output settling. */
static void setup_delay(void)
{
	volatile int i = 10000;
	while(i--);
}



/* As many pins go simultaneously to UEXT40 and UEXT1 we have to do
 * the board test in two phases - with UEXT1, UEXT2 and EXT connectors
 * plugged in, and a second phase with UEXT40 plugged in.
 *
 * As UEXT40 pins 37 and 10 are wired to the same signal (SPI1_NSS), we
 * create two virtual UEXT40 connector descriptions - one with pin 37 and
 * the other one with pin 10.
 */

const struct extcon_board extcon_board_def1 = {
	.name = "STM32-LCD phase1",
	.connectors = extcons_phase1,
	.num_connectors = ARRAY_NUMELEM(extcons_phase1),

	.gpio_read = stm32_gpio_read,
	.gpio_set_inp = stm32_gpio_set_inp,
	.gpio_set_outp = stm32_gpio_set_outp,

	.print_port_pin = print_port_pin,
	.print_con_pin = 0,	/* use default implementation */
	.print_chip_pin = print_chip_pin,
	
	.setup_delay = setup_delay,
};





const struct extcon_board extcon_board_def2 = {
	.name = "STM32-LCD phase2",
	.connectors = extcons_phase2,
	.num_connectors = ARRAY_NUMELEM(extcons_phase2),

	.gpio_read = stm32_gpio_read,
	.gpio_set_inp = stm32_gpio_set_inp,
	.gpio_set_outp = stm32_gpio_set_outp,

	.print_port_pin = print_port_pin,
	.print_con_pin = 0,	/* use default implementation */
	.print_chip_pin = print_chip_pin,
	
	.setup_delay = setup_delay,
};



